{------------------------------------------------------------------------------}
{ TSaveCheckState v2.01 (c) 1998-99 Y-Tech Corporation     February 23rd, 1999 }
{------------------------------------------------------------------------------}
{ TSaveCheckState is a deceptively simple component that can save and restore  }
{ the Checkmarks of all items in any TMainMenu or TPopupMenu component. Menu   }
{ Items can be nested as deep as you want in Submenus. Handles Actions too.    }
{ (Full Source Included)                                                       }
{                                                                              }
{ Please visit our Web Site: http://www.igather.com/components to try out the  }
{ many other freeware and shareware components and classes available there.    }
{------------------------------------------------------------------------------}

{------------------------------------------------------------------------------}
{ Installation                                                                 }
{------------------------------------------------------------------------------}
{ Please register CheckStateReg.pas                                            }
{------------------------------------------------------------------------------}

{------------------------------------------------------------------------------}
{ Questions & Comments                                                         }
{------------------------------------------------------------------------------}
{ All questions and comments should be sent to ycomp@hotpop.com                }
{------------------------------------------------------------------------------}

{------------------------------------------------------------------------------}
{ Usage                                                                        }
{------------------------------------------------------------------------------}
{ - Always read "readme.txt"                                                   }
{ - Please look at "info.htm"                                                  }
{ - Please refer to the help file.                                             }
{------------------------------------------------------------------------------}

{------------------------------------------------------------------------------}
{ Freeware License Agreement & Disclaimer                                      }
{------------------------------------------------------------------------------}
{ By use of this component, you have agreed to the following:                  }
{                                                                              }
{ You may not not modify the source code in this file and re-distribute. You   }
{ may only distribute the unmodified .Zip file that this file you are currently}
{ reading came in. Other than that, you are free to distribute this component. }
{                                                                              }
{ Although this component has been thoroughly tested and documented, neither   }
{ Y-Tech Corporation nor any of it's employees nor the author will be held     }
{ responsible for any damage arising from it's use or misuse.                  }
{------------------------------------------------------------------------------}

{------------------------------------------------------------------------------}
{ Revision History                                                             }
{------------------------------------------------------------------------------}
{ 1.00: + Initial Release                                                      }
{ 1.10: + Added code that raises exception if there's no menu assigned when the}
{         LoadIt or Save methods are called. (How can you save the check states}
{         of menu items without a menu?)                                       }
{ 2.00: + Now released as Freeware with Source.                                }
{       + Fully International Component. You can use this with any version of  }
{         Delphi (except 1.0) with any language. There are no English messages }
{         or prompts shown to the user with this component, so it was easy to  }
{         do.                                                                  }
{ 2.01: + [major bux fix] v2.00 still thought it was Shareware and only worked }
{         under IDE. oops.                                                     }
{------------------------------------------------------------------------------}

unit CheckStateReg;

{$I CompConditionals.inc} // Component Conditional Defines

interface

uses
  Windows, Classes, Registry, Menus;

const
  {$I CompConstants.inc} // Component Constants & Conditional Defines

type
  TCheckAction = (caCheck, caExecute);
  TRegKeyType  = (rtCurrentUser, rtLocalMachine); 

  TSaveCheckState = class(TComponent)
  private
    FMenu: TMenu;
    FCheckAction: TCheckAction;

    FRegKey: String;
    FRegKeyType: TRegKeyType;

    Loaded: Boolean;

    function MenuAssigned: Boolean;

    function GetRootKey: HKEY;
    function AddIfChecked(MenuItem: TMenuItem; Data: Pointer): Boolean;
    function UpdateChecked(MenuItem: TMenuItem; Data: Pointer): Boolean;

  public
    constructor Create(AOwner: TComponent); override;

    procedure Save;
    procedure Load;
    procedure LoadIt;

  published
    property Menu: TMenu read FMenu write FMenu; 
    property CheckAction: TCheckAction read FCheckAction write FCheckAction;

    property RegKey: String read FRegKey write FRegKey;
    property RegKeyType: TRegKeyType read FRegKeyType write FRegKeyType;
  end;

procedure Register;

implementation

uses MenuLib, Dialogs, SysUtils, Forms;

const
  cRegCheckStatesName = 'Check States';
  cNoMenuError        = 'Cannot use the TSaveCheckState component without an assigned Menu property!';

function TSaveCheckState.MenuAssigned: Boolean;
begin
  Result := Assigned(FMenu);
  if not Result then
    raise Exception.Create(cNoMenuError);
end;

function TSaveCheckState.GetRootKey: HKEY;
begin
  Result := HKEY_CURRENT_USER;

  if FRegKeyType = rtLocalMachine then
    Result := HKEY_LOCAL_MACHINE;
end;

function TSaveCheckState.AddIfChecked(MenuItem: TMenuItem; Data: Pointer): Boolean;
begin
  Result := True; // assume success

  with MenuItem do
  try
    if Checked then
      TStringList(Data).Add(Caption); // Save Captions of Each Checked Item

  except
    Result := False;
  end;
end;

function TSaveCheckState.UpdateChecked(MenuItem: TMenuItem; Data: Pointer): Boolean;
var
  Index: Integer;
begin
  Result := True; // assume success

  with MenuItem do
  try
    if TStringList(Data).Find(Caption, Index) then // If Menu Item Caption in List,
      case FCheckAction of
        caCheck  : Checked := True;   // Check It
        caExecute: OnClick(MenuItem); // Execute On Click Code
      end
    else Checked := False; // Otherwise, ensure it is unchecked.

  except
    Result := False;
  end;
end;

procedure TSaveCheckState.Save;
var
  S: TStringList;
begin
  if not MenuAssigned then Exit; // Must have an assigned Menu

  S := TStringList.Create;
  try
    // Get Names of Checked Items
    WalkMenu(FMenu, AddIfChecked, Pointer(S));

    // Save Check States Data
    with TRegistry.Create do
    try
      RootKey := GetRootKey;

      if OpenKey(FRegKey, True) then // If Key Opened or Created Successfully,
        WriteString(cRegCheckStatesName, S.CommaText); // Save Check States Data

    finally
      Free;
    end;

  finally
    S.Free;
  end;
end;

constructor TSaveCheckState.Create(AOwner: TComponent);
begin
  inherited;

  Loaded := False; // Initialize
end;

procedure TSaveCheckState.Load;
begin
  if not Loaded then
    LoadIt;
end;

procedure TSaveCheckState.LoadIt;
var
  S: TStringList;
begin
  if not MenuAssigned then Exit; // Must have an assigned Menu

  S := TStringList.Create;
  try
    with TRegistry.Create do
    try
      RootKey := GetRootKey;

      if OpenKey(FRegKey, False) then // If Data was Saved before,
      begin
        S.CommaText := ReadString(cRegCheckStatesName); // Retrieve it
        S.Sort; // Find will only work on sorted lists

        // Check Items that are in List
        WalkMenu(FMenu, UpdateChecked, Pointer(S));

        // Flag Loaded (even if it really isn't - this prevents multiple load attempts)
        Loaded := True;
      end;

    finally
      Free;
    end;

  finally
    S.Free;
  end;
end;

procedure Register;
begin
  RegisterComponents(cComponentTab, [TSaveCheckState]);
end;

end.
